@@ -13,7 +13,7 @@ use CQL::PrefixNode;
 use CQL::ProxNode;
 use Carp qw( croak );
 
-our $VERSION = '1.0';
+our $VERSION = '1.12';
 
 my $lexer;
 my $token;
@@ -87,7 +87,7 @@ sub new {
     return bless { }, ref($class) || $class;
 }
 
-=head2 parse()
+=head2 parse( $query )
 
 Pass in a CQL query and you'll get back the root node for the CQL parse tree.
 If the CQL is invalid an exception will be thrown.
@@ -114,6 +114,48 @@ sub parse {
     return $root;
 }
 
+=head2 parseSafe( $query )
+
+Pass in a CQL query and you'll get back the root node for the CQL parse tree.
+If the CQL is invalid, an error code from the SRU Diagnostics List 
+will be returned.
+
+=cut
+
+my @cql_errors = (
+    { regex => qr/does not support relational modifiers/,   code => 20 },
+    { regex => qr/expected boolean got /,                   code => 37 },
+    { regex => qr/expected relation modifier got /,         code => 20 },
+    { regex => qr/unknown first-class relation modifier: /, code => 20 },
+    { regex => qr/missing term/,                            code => 27 },
+    { regex => qr/expected proximity relation got /,        code => 40 },
+    { regex => qr/expected proximity distance got /,        code => 41 },
+    { regex => qr/expected proximity unit got/,             code => 42 },
+    { regex => qr/expected proximity ordering got /,        code => 43 },
+    { regex => qr/unknown first class relation: /,          code => 19 },
+    { regex => qr/must supply name/,                        code => 15 },
+    { regex => qr/must supply identifier/,                  code => 15 },
+    { regex => qr/must supply subtree/,                     code => 15 },
+    { regex => qr/must supply term parameter/,              code => 27 },
+    { regex => qr/doesn\'t support relations other than/,   code => 20 },
+);
+
+sub parseSafe {
+    my ($self,$query) = @_;
+
+    my $root = eval { $self->parse( $query ); };
+
+    if ( my $error = $@ ) {
+        my $code = 10;
+        for( @cql_errors ) {
+            $code = $_->{ code } if $error =~ $_->{ regex };
+        }
+        return $code;
+    }
+
+    return $root;
+}
+
 sub parseQuery {
     my ( $qualifier, $relation ) = @_;
     debug( "in parseQuery() with term=" . $token->getString() );
@@ -222,6 +264,7 @@ sub parsePrefix {
 
 sub gatherProxParameters {
     my $node = shift;
+    if (0) {	# CQL 1.0 (obsolete)
     for (my $i=0; $i<4; $i++ ) {
         if ( $token->getType() != CQL_MODIFIER ) { 
             ## end of proximity parameters 
@@ -235,6 +278,28 @@ sub gatherProxParameters {
             elsif ( $i==3 ) { gatherProxOrdering($node); }
         }
     }
+    } else {
+        while ( $token->getType() == CQL_MODIFIER ) {
+	    match( $token );
+	    if ( $token->getType() == CQL_DISTANCE ) {
+		match( $token );
+		gatherProxRelation( $node );
+		gatherProxDistance( $node );
+	    } elsif ( $token->getType() == CQL_UNIT ) {
+		match( $token );
+		if ( $token->getType() != CQL_EQ ) {
+		    croak( "expected proximity unit parameter got ".$token->getString() );
+		}
+		match( $token );
+		gatherProxUnit( $node );
+	    } elsif ( $token->getType() == CQL_ORDERED
+		      || $token->getType() == CQL_UNORDERED ) {
+		gatherProxOrdering( $node );
+	    } else {
+		croak( "expected proximity parameter got ". $token->getString()  ."(". $token->getType() .")" );
+	    }
+        }
+    }
 }
 
 sub gatherProxRelation {
@@ -389,10 +454,19 @@ for more information about Ockham.
 
 =item * Ed Summers - ehs at pobox dot com
 
-=item * Brian Cassidy - brian.cassidy at gmail dot com
+=item * Brian Cassidy - bricas at cpan dot org
 
 =item * Wilbert Hengst - W.Hengst at uva dot nl
 
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 2004-2009 by Ed Summers
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself. 
+
 =cut
 
 1;
